var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};

// src/index.ts
import path3 from "slash-path";

// src/plugin/main.ts
import path2 from "slash-path";
import { build } from "vite";

// src/plugin/write/index.ts
import fs4 from "fs";

// src/plugin/write/handlerFile.ts
import fs from "fs";
var writeHandlerFile = (config, vite) => {
  const { cacheDir, handlerFile, moduleId } = config;
  if (handlerFile.startsWith(cacheDir)) {
    const code = `
import express from "express";
import { applyRouters } from "${moduleId}/routers";

export const handler = express();

applyRouters(
  (props) => {
    const { method, route, path, cb } = props;
    if (handler[method]) {
      handler[method](route, cb);
    } else {
      console.log("Not Support", method, "for", route, "in", handler);
    }
  }
);
`;
    fs.writeFileSync(handlerFile, code);
  }
};

// src/plugin/write/routersFile.ts
import fs2 from "fs";
import { join } from "slash-path";

// src/plugin/write/common.ts
import fg from "fast-glob";
import path from "slash-path";
var assertFileRoute = (...names) => {
  return names.map((name) => name == null ? void 0 : name.replace(/^\//, "").replace(/\/$/, "")).map((name) => name == null ? void 0 : name.replaceAll("$", ":")).map((name) => name == null ? void 0 : name.replaceAll("[", ":").replaceAll("]", "")).map((name) => name == null ? void 0 : name.replace(/\.[^.]+$/, "")).map((name) => name == null ? void 0 : name.replaceAll(/index$/gi, "")).map((name) => name == null ? void 0 : name.replaceAll(/_index$/gi, "")).filter((name) => name).join("/");
};
var getFileRouters = (config) => {
  let { dirs, include, exclude } = config;
  return dirs.flatMap((it, ix) => {
    it.exclude = it.exclude || [];
    const ignore = [...exclude, ...it.exclude];
    return fg.sync(include, {
      ignore,
      onlyDirectories: false,
      dot: true,
      unique: true,
      cwd: it.dir
    }).map((file, jx) => {
      let route = assertFileRoute(it.route, file);
      route = path.join("/", route);
      const pathName = path.join("/", config.routeBase, route);
      file = path.join(it.dir, file);
      const order = route.length * 1e3 + route.split(":").length;
      return {
        name: `_${ix}_${jx}`,
        file,
        path: pathName,
        route,
        order
      };
    }).sort((a, b) => {
      return a.order - b.order;
    });
  });
};
var getMethodRouters = (config) => {
  return getFileRouters(config).flatMap(
    (r) => config.mapperList.map((m) => {
      let cb = r.name + "." + m.name;
      let source = r.file + "?fn=" + m.name;
      source = path.relative(config.root, source);
      return {
        source,
        method: m.method,
        path: r.path,
        route: r.route,
        cb
      };
    })
  );
};

// src/plugin/write/routersFile.ts
var writeRoutersFile = (config, vite) => {
  const { cacheDir, routersFile } = config;
  if (routersFile.startsWith(cacheDir)) {
    const fileRouters = getFileRouters(config);
    const methodRouters = getMethodRouters(config);
    const writeRouter = (c) => {
      return `  ${c.cb} && {
        source: "${c.source}",
        method: "${c.method}",
        route : "${c.route}",
        path  : "${c.path}",
        url   : "${join(vite.base, c.path)}",
        cb    : ${c.cb},
      }`;
    };
    const importFiles = fileRouters.map((it) => `import * as ${it.name} from "/${it.file}";`).join("\n");
    const internalRouter = methodRouters.map((c) => writeRouter(c)).join(",\n");
    const code = `
// Imports
${importFiles}


export const routeBase = "${config.routeBase}";

const internal  = [
${internalRouter}
].filter(it => it);

export const routers = internal.map((it) => { 
  const { method, path, route, url, source} = it;
  return { method, url, path, route, source };
});

export const endpoints = internal.map((it) => it.method?.toUpperCase() + '\\t' + it.url);

const FN = (value) => value;

export const applyRouters = (applyRouter, opts = {} ) => {
  const {pre = FN, post = FN, hoc = FN} = opts;
  pre(internal)
    .forEach((it) => {
    it.cb = hoc(it.cb, it);
    applyRouter(it);
  });  
  post(internal);
};
`;
    fs2.writeFileSync(routersFile, code);
  }
};

// src/plugin/write/serverFile.ts
import fs3 from "fs";
var writeServerFile = (config, vite) => {
  const { cacheDir, serverFile, moduleId } = config;
  if (serverFile.startsWith(cacheDir)) {
    const code = `
import { handler } from "${moduleId}/handler";
import { endpoints } from "${moduleId}/routers";

import express from "express";

const { PORT = 3000, PUBLIC_DIR = "import.meta.env.PUBLIC_DIR" } = process.env;
const server = express();
server.use(express.json());
server.use("import.meta.env.BASE", express.static(PUBLIC_DIR));
server.use("import.meta.env.BASE_AP\xCE", handler);
server.listen(PORT, () => {
  console.log(\`Ready at http://localhost:\${PORT}\`);
  console.log(endpoints);
});
`;
    fs3.writeFileSync(serverFile, code);
  }
};

// src/plugin/write/index.ts
var generateStart = (config, vite) => {
  fs4.mkdirSync(config.cacheDir, { recursive: true });
  writeHandlerFile(config, vite);
  writeServerFile(config, vite);
  generateReload(config, vite);
};
var generateReload = (config, vite) => {
  writeRoutersFile(config, vite);
};

// src/plugin/main.ts
var pluginImpl = (config) => {
  const isReload = (file) => {
    file = path2.slash(file);
    return config.watcherList.find((it) => file.startsWith(it));
  };
  let vite = {};
  return {
    name: "vite-plugin-api",
    enforce: "pre",
    config: () => {
      return {
        resolve: {
          alias: {
            [`${config.moduleId}/server`]: config.serverFile,
            [`${config.moduleId}/handler`]: config.handlerFile,
            [`${config.moduleId}/routers`]: config.routersFile
          }
        }
      };
    },
    configResolved: (viteConfig) => {
      vite = viteConfig;
      generateStart(config, vite);
    },
    handleHotUpdate: async (data) => {
      if (isReload(data.file)) {
        return [];
      }
    },
    configureServer: (devServer) => {
      const {
        //
        watcher,
        middlewares,
        ssrLoadModule,
        ssrFixStacktrace,
        restart
      } = devServer;
      const onReload = (file) => {
        if (isReload(file)) {
          generateReload(config, vite);
          restart();
        }
      };
      watcher.on("add", onReload);
      watcher.on("change", onReload);
      const baseApi = path2.join(vite.base, config.routeBase);
      middlewares.use(baseApi, async (req, res, next) => {
        try {
          const module = await ssrLoadModule(config.handler);
          module.handler(req, res);
        } catch (error) {
          ssrFixStacktrace(error);
          process.exitCode = 1;
          next(error);
        }
      });
    },
    writeBundle: async () => {
      var _a, _b;
      if (process.env.IS_API_BUILD)
        return;
      generateStart(config, vite);
      process.env.IS_API_BUILD = "true";
      const { root, outDir, minify, preBuild, routeBase } = config;
      const publicDir = path2.relative(outDir, vite.build.outDir);
      const viteServer = await preBuild({
        root,
        mode: vite.mode,
        publicDir: "private",
        define: {
          "import.meta.env.PUBLIC_DIR": publicDir,
          "import.meta.env.BASE": vite.base,
          "import.meta.env.BASE_AP\xCE": path2.join(vite.base, routeBase)
        },
        build: {
          outDir,
          ssr: true,
          minify,
          target: "es2020",
          assetsDir: "",
          emptyOutDir: true,
          rollupOptions: {
            input: {
              app: config.server
            },
            external: (_b = (_a = vite.build) == null ? void 0 : _a.rollupOptions) == null ? void 0 : _b.external,
            output: {
              format: "es"
            },
            onwarn: (warning, handler) => {
              if (warning.code === "MISSING_EXPORT" && warning.id === config.routersFile) {
                return;
              }
              handler(warning);
            }
          }
        }
      });
      await build(viteServer);
    }
  };
};

// src/index.ts
var pluginAPI = (opts) => {
  let {
    moduleId = "@api",
    cacheDir = "node_modules/.api",
    root = process.cwd(),
    server = path3.join(cacheDir, "server.js"),
    handler = path3.join(cacheDir, "handler.js"),
    routeBase = "api",
    dirs = [{ dir: "src/api", route: "", exclude: [] }],
    include = ["**/*.ts", "**/*.js"],
    exclude = [],
    mapper = {},
    outDir = "dist/server",
    minify = true,
    preBuild = (v) => v
  } = opts;
  dirs = dirs.map((it) => {
    it.dir = path3.join(root, it.dir);
    return it;
  });
  mapper = __spreadValues({
    default: "use",
    GET: "get",
    PUT: "put",
    POST: "post",
    PATCH: "patch",
    DELETE: "delete"
  }, mapper);
  routeBase = path3.join("/", routeBase);
  outDir = path3.join(root, outDir);
  cacheDir = path3.join(root, cacheDir);
  const serverFile = path3.join(root, server);
  const handlerFile = path3.join(root, handler);
  const routersFile = path3.join(cacheDir, "routers.js");
  const mapperList = Object.entries(mapper).filter((it) => it[1]).map(([name, method]) => {
    return {
      name,
      method
    };
  });
  const watcherList = dirs.map((it) => it.dir);
  watcherList.push(cacheDir);
  watcherList.push(serverFile);
  watcherList.push(handlerFile);
  return pluginImpl({
    moduleId,
    server,
    handler,
    root,
    serverFile,
    handlerFile,
    routersFile,
    routeBase,
    dirs,
    include,
    exclude,
    mapper,
    mapperList,
    watcherList,
    outDir,
    cacheDir,
    minify,
    preBuild
  });
};
var src_default = pluginAPI;
export {
  src_default as default,
  pluginAPI
};
